@@ -26,6 +26,7 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
+#include <gdk/gdkkeysyms.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
@@ -37,12 +38,11 @@
 #include <libgnomeprintui/gnome-print-dialog.h>
 #include <libgnomeprintui/gnome-print-job-preview.h>
 
-#include "gdict.h"
-
 #include "gdict-applet.h"
 #include "gdict-about.h"
 #include "gdict-pref-dialog.h"
 #include "gdict-print.h"
+#include "gdict-common.h"
 
 #include "gtkalignedwindow.h"
 
@@ -117,34 +117,7 @@ static const GtkTargetEntry drop_types[] =
 static const guint n_drop_types = G_N_ELEMENTS (drop_types);
 
 
-/* shows an error dialog making it transient for @parent */
 static void
-show_error_dialog (GtkWindow   *parent,
-		   const gchar *message,
-		   const gchar *detail)
-{
-  GtkWidget *dialog;
-  
-  dialog = gtk_message_dialog_new (parent,
-  				   GTK_DIALOG_DESTROY_WITH_PARENT,
-  				   GTK_MESSAGE_ERROR,
-  				   GTK_BUTTONS_OK,
-  				   "%s", message);
-  gtk_window_set_title (GTK_WINDOW (dialog), "");
-  
-  if (detail)
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-  					      "%s", detail);
-  
-  if (parent && parent->group)
-    gtk_window_group_add_window (parent->group, GTK_WINDOW (dialog));
-  
-  gtk_dialog_run (GTK_DIALOG (dialog));
-  
-  gtk_widget_destroy (dialog);
-}
-
-void
 set_atk_name_description (GtkWidget  *widget,
 			  const char *name,
 			  const char *description)
@@ -274,9 +247,9 @@ save_cb (GtkWidget   *widget,
           
           message = g_strdup_printf (_("Error while writing to '%s'"), filename);
           
-          show_error_dialog (GTK_WINDOW (priv->window),
-          		     message,
-          		     write_error->message);
+          gdict_show_error_dialog (GTK_WINDOW (priv->window),
+          			   message,
+          			   write_error->message);
           
           g_error_free (write_error);
           g_free (message);
@@ -313,6 +286,23 @@ gdict_applet_set_menu_items_sensitive (GdictApplet *applet,
 				NULL);
 }
 
+static gboolean
+window_key_press_event_cb (GtkWidget   *widget,
+			   GdkEventKey *event,
+			   gpointer     user_data)
+{
+  if (event->keyval == GDK_Escape)
+    {
+      GdictApplet *applet = GDICT_APPLET (user_data);
+
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (applet->priv->toggle), FALSE);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 gdict_applet_build_window (GdictApplet *applet)
 {
@@ -331,6 +321,9 @@ gdict_applet_build_window (GdictApplet *applet)
     }
   
   window = gtk_aligned_window_new (priv->toggle);
+  g_signal_connect (window, "key-press-event",
+		    G_CALLBACK (window_key_press_event_cb),
+		    applet);
 
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
@@ -719,16 +712,9 @@ gdict_applet_cmd_preferences (BonoboUIComponent *component,
 			      GdictApplet       *applet,
 			      const gchar       *cname)
 {
-  GdictAppletPrivate *priv = applet->priv;
-  GtkWidget *pref_dialog;
-  
-  pref_dialog = gdict_pref_dialog_new (NULL,
-  				       _("Preferences"),
-  				       priv->loader);
-  
-  gtk_dialog_run (GTK_DIALOG (pref_dialog));
-  
-  gtk_widget_destroy (pref_dialog);
+  gdict_show_pref_dialog (NULL,
+  			  _("Dictionary Preferences"),
+  			  applet->priv->loader);
 }
 
 static void
@@ -754,9 +740,9 @@ gdict_applet_cmd_help (BonoboUIComponent *component,
   					&err);
   if (err)
     {
-      show_error_dialog (NULL,
-      			 _("There was an error while displaying help"),
-      			 err->message);
+      gdict_show_error_dialog (NULL,
+      			       _("There was an error while displaying help"),
+      			       err->message);
       g_error_free (err);
     }
 }
@@ -917,9 +903,9 @@ get_context_from_loader (GdictApplet *applet)
       detail = g_strdup_printf (_("No dictionary source available with name '%s'"),
       				priv->source_name);
 
-      show_error_dialog (NULL,
-                         _("Unable to find dictionary source"),
-                         NULL);
+      gdict_show_error_dialog (NULL,
+                               _("Unable to find dictionary source"),
+                               NULL);
       
       g_free (detail);
 
@@ -937,9 +923,9 @@ get_context_from_loader (GdictApplet *applet)
       detail = g_strdup_printf (_("No context available for source '%s'"),
       				gdict_source_get_description (source));
       				
-      show_error_dialog (NULL,
-                         _("Unable to create a context"),
-                         detail);
+      gdict_show_error_dialog (NULL,
+                               _("Unable to create a context"),
+                               detail);
       
       g_free (detail);
       g_object_unref (source);
@@ -1191,35 +1177,25 @@ static void
 gdict_applet_init (GdictApplet *applet)
 {
   GdictAppletPrivate *priv;
-  gchar *icon_file, *data_dir;
-  GdkPixbuf *icon;
-  GError *icon_error, *gconf_error;
+  gchar *data_dir;
+  GError *gconf_error;
 
   priv = GDICT_APPLET_GET_PRIVATE (applet);
   applet->priv = priv;
       
-  icon_file = g_build_filename (DATADIR,
-		  		"pixmaps",
-				"gnome-dictionary.png",
-				NULL);
-  icon_error = NULL;
-  icon = gdk_pixbuf_new_from_file (icon_file, &icon_error);
-  if (icon_error)
-    {
-      show_error_dialog (NULL,
-			 _("Unable to load the applet icon"),
-			 icon_error->message);
+  data_dir = g_build_filename (g_get_home_dir (),
+  			       ".gnome2",
+  			       "gnome-dictionary",
+  			       NULL);
+  if (!priv->loader)
+    priv->loader = gdict_source_loader_new ();
 
-      g_error_free (icon_error);
-      priv->icon = NULL;
-    }
-  else
-    {
-      gtk_window_set_default_icon (icon);
-      priv->icon = icon;
-    }
+  /* add our data dir inside $HOME to the loader's search paths */
+  gdict_source_loader_add_search_path (priv->loader, data_dir);
 
-  g_free (icon_file);
+  g_free (data_dir);
+  
+  gtk_window_set_default_icon_name ("gnome-dictionary");
   
   panel_applet_set_flags (PANEL_APPLET (applet),
 			  PANEL_APPLET_EXPAND_MINOR);
@@ -1235,11 +1211,9 @@ gdict_applet_init (GdictApplet *applet)
   			&gconf_error);
   if (gconf_error)
     {
-      show_error_dialog (NULL,
-		         _("Unable to connect to GConf"),
-			 gconf_error->message);
-      
-      g_error_free (gconf_error);
+      gdict_show_gerror_dialog (NULL,
+		                _("Unable to connect to GConf"),
+		                gconf_error);
       gconf_error = NULL;
     }
   
@@ -1250,11 +1224,10 @@ gdict_applet_init (GdictApplet *applet)
 					     &gconf_error);
   if (gconf_error)
     {
-      show_error_dialog (NULL,
-		         _("Unable to get notification for preferences"),
-			 gconf_error->message);
+      gdict_show_gerror_dialog (NULL,
+		                _("Unable to get notification for preferences"),
+		                gconf_error);
 
-      g_error_free (gconf_error);
       gconf_error = NULL;
     }
   
@@ -1265,28 +1238,17 @@ gdict_applet_init (GdictApplet *applet)
 						   &gconf_error);
   if (gconf_error)
     {
-      show_error_dialog (NULL,
-			 _("Unable to get notification for the document font"),
-			 gconf_error->message);
+      gdict_show_gerror_dialog (NULL,
+                               _("Unable to get notification for the document font"),
+                               gconf_error);
 
-      g_error_free (gconf_error);
       gconf_error = NULL;
     }
   
-  if (!priv->loader)
-    priv->loader = gdict_source_loader_new ();
-
-  data_dir = g_build_filename (g_get_home_dir (),
-		  	       ".gnome2",
-			       "gnome-dictionary",
-			       NULL);
-
-  /* add our data dir inside $HOME to the loader's search paths */
-  gdict_source_loader_add_search_path (priv->loader, data_dir);
-
-  g_free (data_dir);
-
 #ifndef GDICT_APPLET_STAND_ALONE
+  panel_applet_set_background_widget (PANEL_APPLET (applet),
+		  		      GTK_WIDGET (applet));
+
   priv->size = panel_applet_get_size (PANEL_APPLET (applet));
 
   switch (panel_applet_get_orient (PANEL_APPLET (applet)))
@@ -1308,6 +1270,12 @@ gdict_applet_init (GdictApplet *applet)
   	     priv->size,
   	     (priv->orient == GTK_ORIENTATION_HORIZONTAL ? "H" : "V"));
 #endif /* !GDICT_APPLET_STAND_ALONE */
+
+  priv->icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+		  			 "gnome-dictionary",
+					 48,
+					 0,
+					 NULL);
   
   priv->tooltips = gtk_tooltips_new ();
   g_object_ref (priv->tooltips);
@@ -1366,101 +1334,6 @@ gdict_applet_factory (PanelApplet *applet,
 }
 #endif /* !GDICT_APPLET_STAND_ALONE */
 
-/* create the data directory inside $HOME, if it doesn't exist yet */
-static gboolean
-gdict_applet_create_data_dir (void)
-{
-  gchar *data_dir_name;
-  
-  data_dir_name = g_build_filename (g_get_home_dir (),
-  				    ".gnome2",
-  				    "gnome-dictionary",
-  				    NULL);
-  
-  if (g_mkdir (data_dir_name, 0700) == -1)
-    {
-      /* this is weird, but sometimes there's a "gnome-dictionary" file
-       * inside $HOME/.gnome2; see bug #329126.
-       */
-      if ((errno == EEXIST) &&
-          (g_file_test (data_dir_name, G_FILE_TEST_IS_REGULAR)))
-        {
-          gchar *backup = g_strdup_printf ("%s.pre-2-14", data_dir_name);
-	      
-	  if (g_rename (data_dir_name, backup) == -1)
-	    {
-              GtkWidget *error_dialog;
-
-	      error_dialog = gtk_message_dialog_new (NULL,
-                                                     GTK_DIALOG_MODAL,
-						     GTK_MESSAGE_ERROR,
-						     GTK_BUTTONS_CLOSE,
-						     _("Unable to rename file '%s' to '%s': %s"),
-						     data_dir_name,
-						     backup,
-						     g_strerror (errno));
-	      
-	      gtk_dialog_run (GTK_DIALOG (error_dialog));
-	      
-	      gtk_widget_destroy (error_dialog);
-	      g_free (backup);
-	      g_free (data_dir_name);
-
-	      return FALSE;
-            }
-
-	  g_free (backup);
-	  
-          if (g_mkdir (data_dir_name, 0700) == -1)
-            {
-              GtkWidget *error_dialog;
-	      
-	      error_dialog = gtk_message_dialog_new (NULL,
-						     GTK_DIALOG_MODAL,
-						     GTK_MESSAGE_ERROR,
-						     GTK_BUTTONS_CLOSE,
-						     _("Unable to create the data directory '%s': %s"),
-						     data_dir_name,
-						     g_strerror (errno));
-	      
-	      gtk_dialog_run (GTK_DIALOG (error_dialog));
-	      
-	      gtk_widget_destroy (error_dialog);
-              g_free (data_dir_name);
-
-	      return FALSE;
-            }
-
-	  goto success;
-	}
-      
-      if (errno != EEXIST)
-        {
-          GtkWidget *error_dialog;
-
-	  error_dialog = gtk_message_dialog_new (NULL,
-						 GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Unable to create the data directory '%s': %s"),
-						 data_dir_name,
-						 g_strerror (errno));
-	  
-	  gtk_dialog_run (GTK_DIALOG (error_dialog));
-	  
-	  gtk_widget_destroy (error_dialog);
-	  g_free (data_dir_name);
-
-	  return FALSE;
-	}
-    }
-
-success:
-  g_free (data_dir_name);
-
-  return TRUE;
-}
-
 int
 main (int argc, char *argv[])
 {
@@ -1484,8 +1357,8 @@ main (int argc, char *argv[])
 		      GNOME_PROGRAM_STANDARD_PROPERTIES,
 		      NULL);
 
-  if (!gdict_applet_create_data_dir ())
-    return FALSE;
+  if (!gdict_create_data_dir ())
+    return EXIT_FAILURE;
 
   return panel_applet_factory_main ("OAFIID:GNOME_DictionaryApplet_Factory",
 		                    GDICT_TYPE_APPLET,
@@ -1494,9 +1367,6 @@ main (int argc, char *argv[])
 #else
   gtk_init (&argc, &argv);
 
-  if (!gdict_applet_create_data_dir ())
-    return FALSE;
-  
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL);
 
